using System;
using System.Collections.Generic;
using System.Text;
using Ice;
using System.Threading;
using Demo;

namespace IceLoadBalancing
{
    class Program
    {
        static void Main(string[] args)
        {
            Thread t1 = new Thread(new ThreadStart( delegate()
            {
                HostServer(16000);
            }));
            
            Thread t2 = new Thread( new ThreadStart( delegate
            {
                HostServer(16001);
            }));
            
            t1.Start();
            t2.Start();

            Thread.Sleep(1000 * 2);

            ObjectPrx objProxy = Communicator.stringToProxy("Hello -t:tcp -h localhost -p 16000:tcp -h localhost -p 16001");

            objProxy = objProxy.ice_connectionCached(false);
            objProxy = objProxy.ice_locatorCacheTimeout(0);
            objProxy = objProxy.ice_endpointSelection(Ice.EndpointSelectionType.Random);
            objProxy = objProxy.ice_collocationOptimized(false);

            HelloPrx events = HelloPrxHelper.checkedCast(objProxy);

            for (int i = 0; i < 100; i++)
            {
                //events = HelloPrxHelper.checkedCast(events.ice_connectionId(Guid.NewGuid().ToString()));
                events.sayHello(10);
                
                if (events.ice_getCachedConnection() != null)
                {
                    events.ice_getCachedConnection().close(true);
                }

                if (events.ice_getConnection() != null)
                {
                    events.ice_getConnection().close(true);
                }
            }

            Console.WriteLine("Press Enter key to exit...");
            Console.ReadLine();
        }

        private static Communicator comm_;
        private static object sync_ = new object();
        private static Communicator Communicator
        {
            get
            {
                if (comm_ == null)
                {
                    lock (sync_)
                    {
                        if (comm_ == null)
                        {
                            InitializationData id = new InitializationData();

                            Properties props = Util.createProperties();

                            props.setProperty("Ice.Trace.Network", "3");
                            props.setProperty("Ice.Trace.Retry", "2");
                            props.setProperty("Ice.Trace.Locator", "2");
                            props.setProperty("Ice.Trace.Protocol", "1");
                            props.setProperty("Ice.Trace.Slicing", "1");

                            props.setProperty("Ice.ThreadPool.Client.Size", "1");
                            props.setProperty("Ice.ThreadPool.Client.SizeMax", "25");
                            props.setProperty("Ice.ThreadPool.Server.Size", "10");
                            props.setProperty("Ice.ThreadPool.Server.SizeMax", "25");

                            id.properties = props;

                            comm_ = Util.initialize();
                        }
                    }
                }
                return comm_;
            }
        }

        public static void  HostServer(int port)
        {
            ObjectAdapter adapter = Communicator.createObjectAdapterWithEndpoints("HelloAdapter"+port, "tcp -p " + port);
            Ice.Object obj = new HelloI();
            adapter.add(obj, Ice.Util.stringToIdentity("Hello"));
            adapter.activate();

            Console.WriteLine("HelloI Hosted on Port:->" + port);
            Communicator.waitForShutdown();
        }
    }
}
